from .PQPopupKeypad_widget_1280x800 import PQPopupKeypad_widget_1280x800
from .PQPopupNumpad_simple_widget_1280x800 import PQPopupNumpad_simple_widget_1280x800
from .PQPopupNumpad_widget_1280x800 import PQPopupNumpad_widget_1280x800
from .PQPopupMessage_widget_1280x800 import PQPopupMessage_widget_1280x800